from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.clear_documents_response import ClearDocumentsResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    api_key_header_value: None | str | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_api_key_header_value: None | str | Unset
    if isinstance(api_key_header_value, Unset):
        json_api_key_header_value = UNSET
    else:
        json_api_key_header_value = api_key_header_value
    params["api_key_header_value"] = json_api_key_header_value

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "delete",
        "url": "/documents",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ClearDocumentsResponse | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = ClearDocumentsResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ClearDocumentsResponse | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    api_key_header_value: None | str | Unset = UNSET,
) -> Response[ClearDocumentsResponse | HTTPValidationError]:
    r"""Clear Documents

     Clear all documents from the RAG system.

    This endpoint deletes all documents, entities, relationships, and files from the system.
    It uses the storage drop methods to properly clean up all data and removes all files
    from the input directory.

    Returns:
        ClearDocumentsResponse: A response object containing the status and message.
            - status=\"success\":           All documents and files were successfully cleared.
            - status=\"partial_success\":   Document clear job exit with some errors.
            - status=\"busy\":              Operation could not be completed because the pipeline is
    busy.
            - status=\"fail\":              All storage drop operations failed, with message
            - message: Detailed information about the operation results, including counts
              of deleted files and any errors encountered.

    Raises:
        HTTPException: Raised when a serious error occurs during the clearing process,
                      with status code 500 and error details in the detail field.

    Args:
        api_key_header_value (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ClearDocumentsResponse | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        api_key_header_value=api_key_header_value,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    api_key_header_value: None | str | Unset = UNSET,
) -> ClearDocumentsResponse | HTTPValidationError | None:
    r"""Clear Documents

     Clear all documents from the RAG system.

    This endpoint deletes all documents, entities, relationships, and files from the system.
    It uses the storage drop methods to properly clean up all data and removes all files
    from the input directory.

    Returns:
        ClearDocumentsResponse: A response object containing the status and message.
            - status=\"success\":           All documents and files were successfully cleared.
            - status=\"partial_success\":   Document clear job exit with some errors.
            - status=\"busy\":              Operation could not be completed because the pipeline is
    busy.
            - status=\"fail\":              All storage drop operations failed, with message
            - message: Detailed information about the operation results, including counts
              of deleted files and any errors encountered.

    Raises:
        HTTPException: Raised when a serious error occurs during the clearing process,
                      with status code 500 and error details in the detail field.

    Args:
        api_key_header_value (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ClearDocumentsResponse | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        api_key_header_value=api_key_header_value,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    api_key_header_value: None | str | Unset = UNSET,
) -> Response[ClearDocumentsResponse | HTTPValidationError]:
    r"""Clear Documents

     Clear all documents from the RAG system.

    This endpoint deletes all documents, entities, relationships, and files from the system.
    It uses the storage drop methods to properly clean up all data and removes all files
    from the input directory.

    Returns:
        ClearDocumentsResponse: A response object containing the status and message.
            - status=\"success\":           All documents and files were successfully cleared.
            - status=\"partial_success\":   Document clear job exit with some errors.
            - status=\"busy\":              Operation could not be completed because the pipeline is
    busy.
            - status=\"fail\":              All storage drop operations failed, with message
            - message: Detailed information about the operation results, including counts
              of deleted files and any errors encountered.

    Raises:
        HTTPException: Raised when a serious error occurs during the clearing process,
                      with status code 500 and error details in the detail field.

    Args:
        api_key_header_value (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ClearDocumentsResponse | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        api_key_header_value=api_key_header_value,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    api_key_header_value: None | str | Unset = UNSET,
) -> ClearDocumentsResponse | HTTPValidationError | None:
    r"""Clear Documents

     Clear all documents from the RAG system.

    This endpoint deletes all documents, entities, relationships, and files from the system.
    It uses the storage drop methods to properly clean up all data and removes all files
    from the input directory.

    Returns:
        ClearDocumentsResponse: A response object containing the status and message.
            - status=\"success\":           All documents and files were successfully cleared.
            - status=\"partial_success\":   Document clear job exit with some errors.
            - status=\"busy\":              Operation could not be completed because the pipeline is
    busy.
            - status=\"fail\":              All storage drop operations failed, with message
            - message: Detailed information about the operation results, including counts
              of deleted files and any errors encountered.

    Raises:
        HTTPException: Raised when a serious error occurs during the clearing process,
                      with status code 500 and error details in the detail field.

    Args:
        api_key_header_value (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ClearDocumentsResponse | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            api_key_header_value=api_key_header_value,
        )
    ).parsed
